#include "cpl_string.h"
#include "cpl_error.h"
#include "commonutils.h"
#include "gdal_utils_priv.h"

#include "gdal_priv.h"
#include "vrtdataset.h"

// this function same as SerializeToXML in file vrtrawrasterband.cpp
inline CPLString Trans2String(GIntBig nSize)
{
	char szOffset[22];
	CPLPrintUIntBig(szOffset, nSize, sizeof(szOffset)-1);
	szOffset[sizeof(szOffset)-1] = '\0';

	return CPLString(szOffset);
}

void main()	
{
	CPLSetConfigOption("CPL_DEBUG", "ON");

	int nXSize = 8051;
	int nYSize = 7241;
	int nBandCount = 3;
	int nPixelSize = 2; //Int16
	const char* pszMddFile = "D:\\Hipas\\tm.mdd";	//this is a raw file

	VRTDataset *poVRTDS;
	poVRTDS = new VRTDataset(nXSize, nYSize);
	for( int iBand = 0; iBand < nBandCount; iBand++ )
	{
		char** papszOptions = NULL;
		papszOptions = CSLAddNameValue(papszOptions, "subclass", "VRTRawRasterBand");
		papszOptions = CSLAddNameValue(papszOptions, "SourceFilename", pszMddFile); 
		papszOptions = CSLAddNameValue(papszOptions, "RelativeToVRT", "true");
		papszOptions = CSLAddNameValue(papszOptions, "ByteOrder", "LSB"); 

		GIntBig nImageOffset = 0;
		GIntBig nPixelOffset = 0;
		GIntBig nLineOffset  = 0;

		nImageOffset = nXSize*nYSize*nPixelSize*iBand;
		nPixelOffset = nPixelSize;
		nLineOffset  = nPixelSize*nXSize;

		papszOptions = CSLAddNameValue(papszOptions, "ImageOffset", Trans2String(nImageOffset).c_str()); 
		papszOptions = CSLAddNameValue(papszOptions, "PixelOffset", Trans2String(nPixelOffset).c_str());
		papszOptions = CSLAddNameValue(papszOptions, "LineOffset" , Trans2String(nLineOffset ).c_str()); 

		CPLDebug("TEST", "Band = %d, GIntBig   ImageOffset = %d", iBand, nImageOffset);
		CPLDebug("TEST", "Band = %d, CPLString ImageOffset = %s", iBand, Trans2String(nImageOffset).c_str());

		poVRTDS->AddBand(GDT_Int16, papszOptions);
		CSLDestroy(papszOptions);
	}

	CPLXMLNode *pXML = poVRTDS->SerializeToXML("");
	CPLSerializeXMLTreeToFile(pXML, "D:\\Hipas\\Test.vrt");
	CPLDestroyXMLNode(pXML);

	delete poVRTDS;
}
